@@ -23,10 +23,10 @@
 #ifndef  JOEDOG_PATH_H
 #define  JOEDOG_PATH_H
 
-#define SIEGE_HOME     "/home/jdfulmer"
-#define URL_FILE       "/home/jdfulmer/etc/urls.txt"
-#define CNF_FILE       "/home/jdfulmer/etc/siegerc"
-#define LOG_FILE       "/home/jdfulmer/var/log/siege.log"
+#define SIEGE_HOME     "/home/jeff"
+#define URL_FILE       "/home/jeff/etc/urls.txt"
+#define CNF_FILE       "/home/jeff/etc/siegerc"
+#define LOG_FILE       "/home/jeff/var/log/siege.log"
 #define PLATFORM       "pc-x86_64-linux-gnu"
 
 #endif/*JOEDOG_PATH_H*/
