@@ -62,6 +62,8 @@
 # include <openssl/rand.h>
 #endif/*HAVE_SSL*/
 
+#define MAX_PORT_NO 65535
+
 /** 
  * local prototypes 
  */
@@ -69,6 +71,8 @@ private int     __socket_block(int socket, BOOLEAN block);
 private ssize_t __socket_write(int sock, const void *vbuf, size_t len);  
 private BOOLEAN __socket_check(CONN *C, SDSET mode);
 private BOOLEAN __socket_select(CONN *C, SDSET mode);
+private int     __socket_create(CONN *C, int domain);
+private void   __hostname_strip(char *hn, int len);
 #ifdef  HAVE_POLL
 private BOOLEAN __socket_poll(CONN *C, SDSET mode);
 #endif/*HAVE_POLL*/
@@ -85,16 +89,22 @@ new_socket(CONN *C, const char *hostparam, int portparam)
 {
   int conn;
   int res;
-  int opt;
-  int herrno;
-  struct sockaddr_in cli; 
-  struct hostent     *hp;
+  int addrlen;
+  struct sockaddr *s_addr;
   char   hn[512];
   int    port;
+  int    domain;
 #if defined(__GLIBC__)
-  struct hostent hent;
-  char hbf[8192];
-#elif defined(sun)
+  char port_str[10];
+  struct addrinfo hints;
+  struct addrinfo *addr_res;
+  struct addrinfo *r;
+#else
+  struct sockaddr_in cli;
+  struct hostent     *hp;
+  int herrno;
+#endif
+#if defined(sun)
 # ifndef HAVE_GETIPNODEBYNAME
   struct hostent hent;
   char hbf[8192];
@@ -125,29 +135,33 @@ new_socket(CONN *C, const char *hostparam, int portparam)
     snprintf(hn, sizeof(hn), "%s", hostparam);
     port = portparam;
   }
+  __hostname_strip(hn, 512);
 
-  /* create a socket, return -1 on failure */
-  if ((C->sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-    switch (errno) {
-      case EPROTONOSUPPORT: { NOTIFY(ERROR, "unsupported protocol %s:%d",  __FILE__, __LINE__); break; }
-      case EMFILE:          { NOTIFY(ERROR, "descriptor table full %s:%d", __FILE__, __LINE__); break; }
-      case ENFILE:          { NOTIFY(ERROR, "file table full %s:%d",       __FILE__, __LINE__); break; }
-      case EACCES:          { NOTIFY(ERROR, "permission denied %s:%d",     __FILE__, __LINE__); break; }
-      case ENOBUFS:         { NOTIFY(ERROR, "insufficient buffer %s:%d",   __FILE__, __LINE__); break; }
-      default:              { NOTIFY(ERROR, "unknown socket error %s:%d",  __FILE__, __LINE__); break; }
-    } socket_close(C); return -1;
-  }
-  if (fcntl(C->sock, F_SETFD, O_NDELAY) < 0) {
-    NOTIFY(ERROR, "unable to set close control %s:%d", __FILE__, __LINE__);
+  /* sanity check */
+  if (port < 1 || port > MAX_PORT_NO) {
+    NOTIFY(ERROR, "invalid port number %d in %s:%d", port, __FILE__, __LINE__);
+    return -1;
   }
 
 #if defined(__GLIBC__)
   {
-    memset(hbf, '\0', sizeof hbf);
-    /* for systems using GNU libc */
-    if ((gethostbyname_r(hostparam, &hent, hbf, sizeof(hbf), &hp, &herrno) < 0)) {
-      hp = NULL;
+    snprintf(port_str, sizeof(port_str), "%d", port);
+
+    /* hints for address lookup */
+    memset(&hints, 0, sizeof(struct addrinfo));
+    hints.ai_family=AF_UNSPEC;
+    hints.ai_socktype=SOCK_STREAM;
+    hints.ai_protocol=IPPROTO_TCP;
+
+    res = getaddrinfo(hn, port_str, &hints, &addr_res);
+    if (res != 0) {
+      NOTIFY(ERROR, "Address resolution failed at %s:%d with the following error:", __FILE__, __LINE__);
+      NOTIFY(ERROR, "%s", gai_strerror(res));
+      return -1;
     }
+    s_addr = addr_res->ai_addr;
+    addrlen = addr_res->ai_addrlen;
+    domain = addr_res->ai_family;
   }
 #elif defined(sun)
 # ifdef HAVE_GETIPNODEBYNAME
@@ -174,6 +188,10 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   herrno = h_errno;
 #endif/*OS SPECIFICS*/ 
 
+#if !defined(__GLIBC__)
+  /* gethostbyname only offers IPv4 support */
+  domain = AF_INET;
+
   /**
    * If hp is NULL, then we did not get good information
    * from the name server. Let's notify the user and bail
@@ -199,22 +217,13 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   cli.sin_family = AF_INET;
   cli.sin_port = htons(port);
 
-  if (C->connection.keepalive) {
-    opt = 1; 
-    if (setsockopt(C->sock,SOL_SOCKET,SO_KEEPALIVE,(char *)&opt,sizeof(opt))<0) {
-      switch (errno) {
-        case EBADF:       { NOTIFY(ERROR, "invalid descriptor %s:%d",    __FILE__, __LINE__); break; }
-        case ENOTSOCK:    { NOTIFY(ERROR, "not a socket %s:%d",          __FILE__, __LINE__); break; }
-        case ENOPROTOOPT: { NOTIFY(ERROR, "not a protocol option %s:%d", __FILE__, __LINE__); break; }
-        case EFAULT:      { NOTIFY(ERROR, "setsockopt unknown %s:%d",    __FILE__, __LINE__); break; }
-        default:          { NOTIFY(ERROR, "unknown sockopt error %s:%d", __FILE__, __LINE__); break; }
-      } socket_close(C); return -1;
-    }
-  }
+  s_addr = (struct sockaddr *)&cli;
+  addrlen = sizeof(struct sockaddr_in);
+#endif /* end of __GLIBC__ not defined */
 
-  if ((__socket_block(C->sock, FALSE)) < 0) {
-    NOTIFY(ERROR, "socket: unable to set socket to non-blocking %s:%d", __FILE__, __LINE__);
-    return -1; 
+  /* create a socket, return -1 on failure */
+  if (__socket_create(C, domain) < 0) {
+    return -1;
   }
 
   /**
@@ -222,8 +231,32 @@ new_socket(CONN *C, const char *hostparam, int portparam)
    * evaluate the server response and check for
    * readability/writeability of the socket....
    */ 
-  conn = connect(C->sock, (struct sockaddr *)&cli, sizeof(struct sockaddr_in));
+  conn = connect(C->sock, s_addr, addrlen);
   pthread_testcancel();
+#if defined(__GLIBC__)
+  /**
+    * The result of getaddrinfo is a linked list. Attempt
+    * to connect to each result until successful
+    */
+  if (conn < 0 && errno != EINPROGRESS) {
+    addr_res = addr_res->ai_next;
+    for (r = addr_res; r; r = r->ai_next) {
+      /* close previously opened socket */
+      socket_close(C);
+
+      /* create a socket, return -1 on failure */
+      if (__socket_create(C, domain) < 0) {
+        return -1;
+      }
+
+      conn = connect(C->sock, s_addr, addrlen);
+      pthread_testcancel();
+      if (conn == 0) {
+        break;
+      }
+    }
+  }
+#endif
   if (conn < 0 && errno != EINPROGRESS) {
     switch (errno) {
       case EACCES:        {NOTIFY(ERROR, "socket: %d EACCES",                  pthread_self()); break;}
@@ -244,7 +277,7 @@ new_socket(CONN *C, const char *hostparam, int portparam)
       /**
        * If we reconnect and receive EISCONN, then we have a successful connection
        */
-      res = connect(C->sock, (struct sockaddr *)&cli, sizeof(struct sockaddr_in)); 
+      res = connect(C->sock, s_addr, addrlen);
       if((res < 0)&&(errno != EISCONN)){
         NOTIFY(ERROR, "socket: unable to connect %s:%d", __FILE__, __LINE__);
         socket_close(C);
@@ -356,6 +389,69 @@ __socket_select(CONN *C, SDSET mode)
   }
 }
 
+/**
+ * Create new socket and set socket options.
+ * Handle possible error codes.
+ */
+private int
+__socket_create(CONN *C, int domain)
+{
+  if ((C->sock = socket(domain, SOCK_STREAM, 0)) < 0) {
+    switch (errno) {
+      case EPROTONOSUPPORT: { NOTIFY(ERROR, "unsupported protocol %s:%d",  __FILE__, __LINE__); break; }
+      case EMFILE:          { NOTIFY(ERROR, "descriptor table full %s:%d", __FILE__, __LINE__); break; }
+      case ENFILE:          { NOTIFY(ERROR, "file table full %s:%d",       __FILE__, __LINE__); break; }
+      case EACCES:          { NOTIFY(ERROR, "permission denied %s:%d",     __FILE__, __LINE__); break; }
+      case ENOBUFS:         { NOTIFY(ERROR, "insufficient buffer %s:%d",   __FILE__, __LINE__); break; }
+      default:              { NOTIFY(ERROR, "unknown socket error %s:%d",  __FILE__, __LINE__); break; }
+    } socket_close(C); return -1;
+  }
+  if (fcntl(C->sock, F_SETFD, O_NDELAY) < 0) {
+    NOTIFY(ERROR, "unable to set close control %s:%d", __FILE__, __LINE__);
+  }
+
+  if (C->connection.keepalive) {
+    int opt = 1;
+    if (setsockopt(C->sock,SOL_SOCKET,SO_KEEPALIVE,(char *)&opt,sizeof(opt))<0) {
+      switch (errno) {
+        case EBADF:       { NOTIFY(ERROR, "invalid descriptor %s:%d",    __FILE__, __LINE__); break; }
+        case ENOTSOCK:    { NOTIFY(ERROR, "not a socket %s:%d",          __FILE__, __LINE__); break; }
+        case ENOPROTOOPT: { NOTIFY(ERROR, "not a protocol option %s:%d", __FILE__, __LINE__); break; }
+        case EFAULT:      { NOTIFY(ERROR, "setsockopt unknown %s:%d",    __FILE__, __LINE__); break; }
+        default:          { NOTIFY(ERROR, "unknown sockopt error %s:%d", __FILE__, __LINE__); break; }
+      } socket_close(C); return -1;
+    }
+  }
+
+  if ((__socket_block(C->sock, FALSE)) < 0) {
+    NOTIFY(ERROR, "socket: unable to set socket to non-blocking %s:%d", __FILE__, __LINE__);
+    return -1;
+  }
+
+  return 0;
+}
+
+/**
+ * remove square bracket
+ * around IPv6 addresses
+ */
+private void
+__hostname_strip(char *hn, int len)
+{
+  int i;
+
+  if (startswith("[", hn)) {
+    memmove(hn, hn + 1, len - 1);
+
+    /* skip to matching square bracket */
+    for (i = 0; hn[i] && hn[i] != ']'; i++);
+
+    if (hn[i] == ']') {
+      memmove(hn + i, hn + i + 1, len - i - 1);
+    }
+  }
+}
+
 /**
  * local function
  * set socket to non-blocking
